module Monark

  # Monark runtime. Encapsulates state and manages migrations.
  class Context
    attr_accessor :root, :migrations_file_name, :connection
    attr_reader :migrations, :version
    attr_writer :manifest
    attr_writer :env

    def initialize(root, migrations_file_name = Monark::DEFAULT_MIGRATIONS_FILE_NAME)
      self.root                 = Pathname.new(root)
      self.migrations_file_name = migrations_file_name
      @migrations               = []
    end

    def env
      defined?(@env) ? @env : Monark.env
    end

    def manifest
      @manifest ||= root.join(migrations_file_name).read
    end

    def version=(number)
      @version = Integer(number)
    end

    def load_manifest
      @loaded_manifest ||= begin
        DSL.new(self).instance_eval(manifest, root.join(migrations_file_name).to_s, 0)
        true
      end
    end

    def push(*args)
      migrations.push(*args)
    end

    def done
      load_manifest
      count = migrations.count
      schema = Schema.new
      schema.next_migration = count if count > schema.next_migration
    end

    def assume_done
      load_manifest
      schema = Schema.new
      schema.next_migration = migrations.count
    end

    def migrate(options = {})
      load_manifest
      schema = Schema.new
      ActiveRecord::Base.transaction do
        migrations.each_with_index do |migration, index|
          if (index >= schema.next_migration) || (migration.reloadable? && options[:reload])
            migration.perform
          end
        end
        done
      end
    end
  end
end
