module Monark

  # A Monark migration. Acceptable formats are `.sql`, `.sql.erb` and `.rb`.
  class Migration

    attr_accessor :root, :path, :kind, :reloadable
    alias reloadable? reloadable

    def initialize(root, path, proc = nil)
      self.root = Pathname(root.to_s)
      self.path = path
      @code = proc
    end

    def code
      @code ||= load_file("sql", "sql.erb", "rb")
    end

    def perform
      code
      if !kind
        puts "-> calling '#{@path}'"
        @code.call
      elsif @kind == "rb"
        puts "-> evaluating '#{@path}'"
        binding.eval(@code, path, 0)
      else
        puts "-> executing '#{@path}'"
        ActiveRecord::Base.connection.execute @code
      end
    end

    private

    def load_file(*extensions)
      extensions.each do |ext|
        file = root.join("#{path}.#{ext}")
        if file.readable?
          data = file.read
          @kind = ext.gsub(/\.erb\z/, '')
          return ext =~ /\.erb\z/ ? ERB.new(data).result(binding) : data
        end
      end
    end
  end
end