module Monark

  # The Monark schema. Creates and updates the schema in an SQL database.
  class Schema

    attr_accessor :context
    attr_reader :version, :next_migration

    def initialize
      load_info
    end

    def next_migration=(value)
      value = Integer(value)
      conn.execute "UPDATE monark_info SET next_migration = #{value}"
      @next_migration = value
    end

    private

    def create_table_if_not_exists
      conn.execute 'CREATE TABLE IF NOT EXISTS monark_info(version integer NOT NULL, next_migration integer NOT NULL)'
    end

    def get_first_row
      result = conn.select_all 'SELECT version, next_migration FROM monark_info LIMIT 1'
      if result.empty?
        @version        = 1
        @next_migration = 0
        conn.execute "INSERT INTO monark_info (version, next_migration) VALUES (#{@version}, #{@next_migration})"
      else
        @version = Integer(result[0]['version'])
        @next_migration = Integer(result[0]['next_migration'])
      end
    end

    def load_info
      conn.transaction do
        create_table_if_not_exists
        get_first_row
      end
    end

    def conn
      ActiveRecord::Base.connection
    end
  end
end