require 'rake'

namespace :monark do

  def monark_root
    @monark_root ||= Rails.root.join("db/monark")
  end

  desc 'Perform pending migrations'
  task :migrate => :environment do
    context = Monark::Context.new monark_root
    context.migrate
  end

  desc 'Perform reloadable and pending migrations'
  task :reload => :environment do
    context = Monark::Context.new monark_root
    context.migrate(:reload => true)
  end

  desc 'Perform all migrations again'
  task :reset => :environment do
    ActiveRecord::Base.connection.transaction do
      schema = Monark::Schema.new
      schema.next_migration = 0
      Rake::Task['monark:migrate'].invoke
    end
  end

  desc 'Assume all migrations have been performed'
  task :assume_done => :environment do
    context = Monark::Context.new monark_root
    context.assume_done
    schema = Monark::Schema.new
    puts "Next migration set to #{schema.next_migration}"
  end
end